#include <assert.h>

#include "nrf_drv_spi.h"
#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"

#include "driver.h"
#include "spi.h"

#define SPI_INSTANCE    0
static const nrf_drv_spi_t spi = NRF_DRV_SPI_INSTANCE(SPI_INSTANCE);
static volatile bool spi_xfer_done;

struct spi_object {
    uint8_t *rx_buf;
    uint8_t *tx_buf;
    uint32_t rx_len;
    uint32_t tx_len;
};

uint8_t rx_buffer[1];
uint8_t tx_buffer[20];
static struct spi_object spi_obj = {
    .rx_buf = rx_buffer,
    .tx_buf = tx_buffer,
    .rx_len = sizeof(rx_buffer),
    .tx_len = sizeof(tx_buffer)
};

void spi_event_handler(nrf_drv_spi_evt_t const * p_event, void * p_context)
{
    spi_xfer_done = true;
    NRF_LOG_INFO("Transfer completed.");
}

int spi_open(const struct driver *drv)
{
    assert(NULL != drv);

    struct spi *this = (struct spi *)(drv->dev);

    nrf_drv_spi_config_t spi_config = NRF_DRV_SPI_DEFAULT_CONFIG;
    spi_config.ss_pin = this->ss_pin;
    spi_config.miso_pin = this->miso_pin;
    spi_config.mosi_pin = this->mosi_pin;
    spi_config.sck_pin  = this->sck_pin;
    APP_ERROR_CHECK(nrf_drv_spi_init(&spi, &spi_config, spi_event_handler, NULL));

    return 0;
}

int spi_close(const struct driver *drv)
{
    assert(NULL != drv);

    return 0;
}

int spi_read(const struct driver *drv, char *buffer, unsigned int len)
{
    assert((NULL != drv) && (NULL != buffer));

    NRF_LOG_INFO("spi read not implemented yet.");

    return 0;
}

int spi_write(const struct driver *drv, const char *buffer, unsigned int len)
{
    if(len > spi_obj.tx_len) {
        NRF_LOG_ERROR("buffer too small");
        return 0;
    }
    memcpy(spi_obj.tx_buf, buffer, len);

    // Reset transfer done flag
    spi_xfer_done = false;

    APP_ERROR_CHECK(nrf_drv_spi_transfer(&spi, spi_obj.tx_buf, spi_obj.tx_len, spi_obj.rx_buf, spi_obj.rx_len));

    while(!spi_xfer_done) {
        __WFE();
    }
    return (int)len;
}
